//---------------------------------------------------------------------------
#include <vcl.h>
#include <tchar.h>
#pragma hdrstop

#include "LED1Panel.h"
#pragma package(smart_init)
#pragma resource "LED1Panel.dcr"
#pragma link "_GClass"
#pragma link "AbLED"

// in C++ the array is zero based so the [0] entry is invalid, in Lua the array is one based
static String AspectNamesSource[] = {"ZERO", "None", "Dark", "Stop Signal", "Clear", "Stop and Proceed", "Approach", "Medium Clear",
	"Medium Approach", "Slow Clear", "Slow Approach", "Approach Medium", "Approach Slow",
	"Restricting", "Approach Limited", "Limited Clear", "Limited Approach", "Medium-Approach Slow"};
static int AspectCount = 18;
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TLED1Panel *)
{
	new TLED1Panel(NULL);
}
//---------------------------------------------------------------------------
__fastcall TLED1Panel::TLED1Panel(TComponent* Owner)
	: TPanel(Owner)
{

/*
Under some circumstances, you may need to register TShape, etc.
This event would normally occur when you drop a component on a form.
However, in this case a TShape, etc is not necessarily ever dropped
explicitly on a form. As a result, the safe thing to do is register
the components in the constructor for the class.
*/

TComponentClass classes[5]={__classid(TLabel),  __classid(TShape), __classid(TAbLED), __classid(TLED1Panel), __classid(TPanel)};
RegisterClasses(classes, 4);

for (int i = 0 ; i < AspectCount ; i++) {
	AspectNames[i] = AspectNamesSource[i];
    }

Width = 27;
Height = 105-40;
Caption = "";
Name = "LED1Panel";
Color = clHighlightText;
Hint = "1 Light Signal Head displays speed indication";
ShowHint = true;

SH = new TShape(this);
SH->Parent = this;

SH->Brush->Color = clBlack;
SH->Top = 0;
SH->Left = 0;
SH->Width = 27;
SH->Height = 105 - 40;
SH->Shape = stRoundRect;

SigNam = new TLabel(this);
NumPlate = new TLabel(this);

SigNam->Parent = this;
NumPlate->Parent = this;

SigNam->AutoSize = false;
SigNam->Color = clBlack;
SigNam->Font->Color = clWhite;
SigNam->Top = 2;
SigNam->Left = 0;
SigNam->Width = 27;
SigNam->Height = 14;
SigNam->Hint = "Signal Name";
SigNam->ShowHint = true;
SigNam->AutoSize = false;		// no effect
SigNam->Font->Style = TFontStyles() << fsBold;

NumPlate->AutoSize = false;
NumPlate->Color = clBlack;
NumPlate->Font->Color = clWhite;
NumPlate->Top = 85 - 40;
NumPlate->Left = 0;
NumPlate->Width = 27;
NumPlate->Height = 14;
NumPlate->Hint = "Permissive";
NumPlate->ShowHint = true;
NumPlate->AutoSize = false;	// no effect
NumPlate->Visible = false;

SigNam->Caption = "-----";
SigNam->Alignment = taCenter;

NumPlate->Caption = "";
NumPlate->Alignment = taCenter;

LED1 = new TAbLED(this);

LED1->Parent = this;

LED1->Left = 4;

LED1->Top = 12;

LED1->LED->Width = 20;
LED1->LED->Height = 20;

LED1->LED->ColorOn = clLime;

LED1->LED->ColorOff = clBlack;

LED1->Hint = "High Speed Head";

LED1->ShowHint = true;

LED1->Name = "HighSpeedHead";

}
//---------------------------------------------------------------------------
__fastcall TLED1Panel::~TLED1Panel(void)
{
delete SH;
delete SigNam;
delete NumPlate;
delete LED1;
}//---------------------------------------------------------------------------

void __fastcall TLED1Panel::SetParent(TWinControl *AParent)
{
TPanel::SetParent(AParent);
Caption = "";

#if __BORLANDC__ < 0x720
// Defined for CB10.1 C++ programs that use the VCL.
LED1->Checked = true;
#endif
}
//---------------------------------------------------------------------------

void __fastcall TLED1Panel::Loaded()
{
TPanel::Loaded();
}
//---------------------------------------------------------------------------

bool __fastcall TLED1Panel::Lights(int Aspect)
{
if (Aspect == AsNone) return false;	// do nothing
Tag = Aspect;	// put the Aspect into the Tag
LEDHighSpeed->Flashing = false;
switch(Aspect) {
    case AsDark:	// Off
        LEDHighSpeed->LED->ColorOn = clBlack;
        Hint = AspectNames[AsDark];
        LEDHighSpeed->Hint = "Dark";
        break;
    case AsClear:	// Clear - Rule 281
        LEDHighSpeed->LED->ColorOn = clLime;
        Hint = AspectNames[AsClear];
        LEDHighSpeed->Hint = "Proceed";
        break;
    case AsApproach:	// Approach - Rule 285
        LEDHighSpeed->LED->ColorOn = clYellow;
        Hint = AspectNames[AsApproach];
        LEDHighSpeed->Hint = "Proceed prepared to stop at next signal. Train exceeding Medium Speed must at once reduce to that speed.";
        break;
    case AsStopandProceed:	// Stop and Proceed - Rule 291
        LEDHighSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsStopandProceed];
        LEDHighSpeed->Hint = "Stop: Then proceed at Restricting Speed";
        break;
    case AsStopSignal:	// Stop - Rule 292
        LEDHighSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsStopSignal];
        LEDHighSpeed->Hint = "Stop";
        break;
    default:
        LEDHighSpeed->LED->ColorOn = clBlack;
        String Msg = "Aspect: " + AspectNames[Aspect];
        Application->MessageBox(_TEXT("Invalid Signal Aspect for 1 light signal Requested. "), Msg.c_str(), MB_OK);
        Hint = "Invalid Aspect";
        LEDHighSpeed->Hint = "Stop and determine cause of invalid aspect.";
        break;
    }
SignalName->Hint = "1-Light Signal - " + Name + " : " + Hint;
NumberPlate->Hint = Hint;
return true;
}
//---------------------------------------------------------------------------

bool __fastcall TLED1Panel::Names(String sName, int iNumberPlate, int iType)
{
// sName == "" means don't change the Signal Name
// iNumberPlate == -1 means don't change the Number Plate
// iNumberPlate == 0 means make the Number Plate "A"
// iType == -1 means don't change the Signal Type
if (sName != "") SignalName->Caption = sName;
if (iNumberPlate == 0) {
    NumberPlate->Caption = "A";
    NumberPlate->Visible = true;
    }
if (iNumberPlate > 0) {
	NumberPlate->Caption = String(iNumberPlate);
   	NumberPlate->Visible = true;
    }
if (iType != -1) {
   	Constraints->MinWidth = iType;	// free spot to store the Signal Type for Cab Signals
    }
return true;
}
//---------------------------------------------------------------------------

namespace Led1panel
{
	void __fastcall PACKAGE Register()
	{
		TComponentClass classes[1] = {__classid(TLED1Panel)};
		RegisterComponents("RTC", classes, 0);
	}
}
//---------------------------------------------------------------------------

