//---------------------------------------------------------------------------
#include <vcl.h>
#include <tchar.h>
#pragma hdrstop

#include "LED3Panel.h"
#pragma package(smart_init)
#pragma resource "LED3Panel.dcr"
#pragma link "_GClass"
#pragma link "AbLED"

// in C++ the array is zero based so the [0] entry is invalid, in Lua the array is one based
static String AspectNamesSource[] = {"ZERO", "None", "Dark", "Stop Signal", "Clear", "Stop and Proceed", "Approach", "Medium Clear",
	"Medium Approach", "Slow Clear", "Slow Approach", "Approach Medium", "Approach Slow",
	"Restricting", "Approach Limited", "Limited Clear", "Limited Approach", "Medium-Approach Slow"};
static int AspectCount = 18;
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TLED3Panel *)
{
	new TLED3Panel(NULL);
}
//---------------------------------------------------------------------------
__fastcall TLED3Panel::TLED3Panel(TComponent* Owner)
	: TPanel(Owner)
{

/*
Under some circumstances, you may need to register TShape, etc.
This event would normally occur when you drop a component on a form.
However, in this case a TShape, etc is not necessarily ever dropped
explicitly on a form. As a result, the safe thing to do is register
the components in the constructor for the class.
*/

TComponentClass classes[5]={__classid(TLabel),  __classid(TShape), __classid(TAbLED), __classid(TLED3Panel), __classid(TPanel)};
RegisterClasses(classes, 4);

for (int i = 0 ; i < AspectCount ; i++) {
	AspectNames[i] = AspectNamesSource[i];
    }

Width = 27;
Height = 105;
Caption = "";
Name = "LED3Panel";
Color = clHighlightText;
Hint = "3 Light Signal Head displays high, medium and low speed indications";
ShowHint = true;

SH = new TShape(this);
SH->Parent = this;

SH->Brush->Color = clBlack;
SH->Top = 0;
SH->Left = 0;
SH->Width = 27;
SH->Height = 105;
SH->Shape = stRoundRect;

SigNam = new TLabel(this);
NumPlate = new TLabel(this);

SigNam->Parent = this;
NumPlate->Parent = this;

SigNam->AutoSize = false;
SigNam->Color = clBlack;
SigNam->Font->Color = clWhite;
SigNam->Top = 2;
SigNam->Left = 0;
SigNam->Width = 27;
SigNam->Height = 14;
SigNam->Hint = "Signal Name";
SigNam->ShowHint = true;
SigNam->AutoSize = false;		// no effect
SigNam->Font->Style = TFontStyles() << fsBold;

NumPlate->AutoSize = false;
NumPlate->Color = clBlack;
NumPlate->Font->Color = clWhite;
NumPlate->Top = 85;
NumPlate->Left = 0;
NumPlate->Width = 27;
NumPlate->Height = 14;
NumPlate->Hint = "Permissive";
NumPlate->ShowHint = true;
NumPlate->AutoSize = false;	// no effect
NumPlate->Visible = false;

SigNam->Caption = "-----";
SigNam->Alignment = taCenter;

NumPlate->Caption = "";
NumPlate->Alignment = taCenter;

LED1 = new TAbLED(this);
LED2 = new TAbLED(this);
LED3 = new TAbLED(this);

LED1->Parent = this;
LED2->Parent = this;
LED3->Parent = this;

LED1->Left = 4;
LED2->Left = 4;
LED3->Left = 4;

LED1->Top = 12;
LED2->Top = 34;
LED3->Top = 56;

LED1->LED->Width = 20;
LED1->LED->Height = 20;
LED2->LED->Width = 20;
LED2->LED->Height = 20;
LED3->LED->Width = 20;
LED3->LED->Height = 20;

LED1->LED->ColorOn = clLime;
LED2->LED->ColorOn = clYellow;
LED3->LED->ColorOn = clRed;

LED1->LED->ColorOff = clBlack;
LED2->LED->ColorOff = clBlack;
LED3->LED->ColorOff = clBlack;

LED1->Hint = "High Speed Head";
LED2->Hint = "Medium Speed Head";
LED3->Hint = "Low Speed Head";

LED1->ShowHint = true;
LED2->ShowHint = true;
LED3->ShowHint = true;

LED1->Name = "HighSpeedHead";
LED2->Name = "MediumSpeedHead";
LED3->Name = "LowSpeedHead";

}
//---------------------------------------------------------------------------
__fastcall TLED3Panel::~TLED3Panel(void)
{
delete SH;
delete SigNam;
delete NumPlate;
delete LED1;
delete LED2;
delete LED3;
}//---------------------------------------------------------------------------

void __fastcall TLED3Panel::SetParent(TWinControl *AParent)
{
TPanel::SetParent(AParent);
Caption = "";

#if __BORLANDC__ < 0x720
// Defined for BCB4.0 C++ programs that use the VCL.
LED1->Checked = true;
LED2->Checked = true;
LED3->Checked = true;
#endif
}
//---------------------------------------------------------------------------

void __fastcall TLED3Panel::Loaded()
{
TPanel::Loaded();
}
//---------------------------------------------------------------------------

bool __fastcall TLED3Panel::Lights(int Aspect)
{
if (Aspect == AsNone) return false;	// do nothing
Tag = Aspect;	// put the Aspect into the Tag
LEDHighSpeed->Flashing = false;
LEDMediumSpeed->Flashing = false;
LEDLowSpeed->Flashing = false;
switch(Aspect) {
    case AsDark:	// Dark
        LEDHighSpeed->LED->ColorOn = clBlack;
        LEDMediumSpeed->LED->ColorOn = clBlack;
        LEDLowSpeed->LED->ColorOn = clBlack;
        Hint = AspectNames[AsDark];
        LEDHighSpeed->Hint = "Dark";
        break;
    case AsClear:	// Clear - Rule 281
        LEDHighSpeed->LED->ColorOn = clLime;
        LEDMediumSpeed->LED->ColorOn = clRed;
        LEDLowSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsClear];
        LEDHighSpeed->Hint = "Proceed";
        break;
    case AsApproachLimited:	// Approach Limited - Rule 281(B)
        LEDHighSpeed->LED->ColorOn = clYellow;
        LEDMediumSpeed->Flashing = true;
        LEDMediumSpeed->LED->ColorOn = clLime;
        LEDLowSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsApproachLimited];
        LEDHighSpeed->Hint = "Proceed approaching next signal at Limited Speed.";
        break;
    case AsLimitedClear:	// Limited Clear - Rule 281(C)
        LEDHighSpeed->LED->ColorOn = clRed;
        LEDMediumSpeed->Flashing = true;
        LEDMediumSpeed->LED->ColorOn = clLime;
        LEDLowSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsLimitedClear];
        LEDHighSpeed->Hint = "Proceed: Limited Speed within interlocking limits.";
        break;
    case AsLimitedApproach:	// Limited Approach - Rule 281(D)
        LEDHighSpeed->LED->ColorOn = clRed;
        LEDMediumSpeed->Flashing = true;
        LEDMediumSpeed->LED->ColorOn = clYellow;
        LEDLowSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsLimitedApproach];
        LEDHighSpeed->Hint = "Proceed at Limited Speed preparing to stop at next signal.";
        break;
    case AsApproachMedium:	// Approach Medium - Rule 282
        LEDHighSpeed->LED->ColorOn = clYellow;
        LEDMediumSpeed->LED->ColorOn = clLime;
        LEDLowSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsApproachMedium];
        LEDHighSpeed->Hint = "Proceed approaching next signal at Medium Speed.";
        break;
    case AsMediumClear:	// Medium Clear - Rule 283
        LEDHighSpeed->LED->ColorOn = clRed;
        LEDMediumSpeed->LED->ColorOn = clLime;
        LEDLowSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsMediumClear];
        LEDHighSpeed->Hint = "Proceed: Medium Speed within interlocking limits.";
        break;
    case AsMediumApproachSlow:	// Medium-Approach Slow - Rule 283(B)
        LEDHighSpeed->LED->ColorOn = clRed;
        LEDMediumSpeed->LED->ColorOn = clYellow;
        LEDLowSpeed->LED->ColorOn = clGreen;
        Hint = AspectNames[AsMediumApproachSlow];
        LEDHighSpeed->Hint = "Proceed at Medium Speed approaching next signal at slow speed.";
        break;
    case AsApproachSlow:	// Approach Slow - Rule 284
        LEDHighSpeed->LED->ColorOn = clYellow;
        LEDMediumSpeed->LED->ColorOn = clRed;
        LEDLowSpeed->LED->ColorOn = clLime;
        Hint = AspectNames[AsApproachSlow];
        LEDHighSpeed->Hint = "Proceed approaching next signal at Slow Speed. Train exceeding Medium Speed must at once reduce to that speed.";
        break;
    case AsApproach:	// Approach - Rule 285
        LEDHighSpeed->LED->ColorOn = clYellow;
        LEDMediumSpeed->LED->ColorOn = clRed;
        LEDLowSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsApproach];
        LEDHighSpeed->Hint = "Proceed prepared to stop at next signal. Train exceeding Medium Speed must at once reduce to that speed.";
        break;
    case AsMediumApproach:	// Medium-Approach - Rule 286
        LEDHighSpeed->LED->ColorOn = clRed;
        LEDMediumSpeed->LED->ColorOn = clYellow;
        LEDLowSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsMediumApproach];
        LEDHighSpeed->Hint = "Proceed at Medium Speed preparing to stop at next signal.";
        break;
    case AsSlowClear:	// Slow-Clear - Rule 287
        LEDHighSpeed->LED->ColorOn = clRed;
        LEDMediumSpeed->LED->ColorOn = clRed;
        LEDLowSpeed->LED->ColorOn = clLime;
        Hint = AspectNames[AsSlowClear];
        LEDHighSpeed->Hint = "Proceed: Slow Speed within interlocking limits.";
        break;
    case AsSlowApproach:	// Slow-Approach - Rule 288
        LEDHighSpeed->LED->ColorOn = clRed;
        LEDMediumSpeed->LED->ColorOn = clRed;
        LEDLowSpeed->Flashing = true;
        LEDLowSpeed->LED->ColorOn = clYellow;
        Hint = AspectNames[AsSlowApproach];
        LEDHighSpeed->Hint = "Proceed preparing to stop at next signal: Slow Speed within interlocking limits.";
        break;
    case AsRestricting:		// Restricting - Rule 290
        LEDHighSpeed->LED->ColorOn = clRed;
        LEDMediumSpeed->LED->ColorOn = clRed;
        LEDLowSpeed->LED->ColorOn = clYellow;
        Hint = AspectNames[AsRestricting];
        LEDHighSpeed->Hint = "Proceed at Restricted Speed.";
        break;
    case AsStopandProceed:	// Stop and Proceed - Rule 291
        LEDHighSpeed->LED->ColorOn = clRed;
        LEDMediumSpeed->LED->ColorOn = clRed;
        LEDLowSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsStopandProceed];
        LEDHighSpeed->Hint = "Stop: Then proceed at Restricting Speed.";
        break;
   case AsStopSignal:	// Stop Signal - Rule 292
        LEDHighSpeed->LED->ColorOn = clRed;
        LEDMediumSpeed->LED->ColorOn = clRed;
        LEDLowSpeed->LED->ColorOn = clRed;
        Hint = AspectNames[AsStopSignal];
        LEDHighSpeed->Hint = "Stop";
        break;
   default:
        LEDHighSpeed->LED->ColorOn = clBlack;
        LEDMediumSpeed->LED->ColorOn = clBlack;
        LEDLowSpeed->LED->ColorOn = clBlack;        
        String Msg = "Aspect: " + AspectNames[Aspect];
        Application->MessageBox(_TEXT("Invalid Signal Aspect for 3 light signal Requested. "), Msg.c_str(), MB_OK);
        Hint = "Invalid Aspect";
        LEDHighSpeed->Hint = "Stop and determine cause of invalid aspect.";
        break;
    }
SignalName->Hint = "3-Light Signal - " + Name + " : " + Hint;
NumberPlate->Hint = Hint;
LEDMediumSpeed->Hint = LEDHighSpeed->Hint;
LEDLowSpeed->Hint = LEDHighSpeed->Hint;
return true;
}
//---------------------------------------------------------------------------

bool __fastcall TLED3Panel::Names(String sName, int iNumberPlate, int iType)
{
// sName == "" means don't change the Signal Name
// iNumberPlate == -1 means don't change the Number Plate
// iNumberPlate == 0 means make the Number Plate "A"
// iType == -1 means don't change the Signal Type
if (sName != "") SignalName->Caption = sName;
if (iNumberPlate == 0) {
    NumberPlate->Caption = "A";
    NumberPlate->Visible = true;
    }
if (iNumberPlate > 0) {
	NumberPlate->Caption = String(iNumberPlate);
   	NumberPlate->Visible = true;
    }
if (iType != -1) {
   	Constraints->MinWidth = iType;	// free spot to store the Signal Type for Cab Signals
    }
return true;
}
//---------------------------------------------------------------------------
namespace Led3panel
{
	void __fastcall PACKAGE Register()
	{
		TComponentClass classes[1] = {__classid(TLED3Panel)};
		RegisterComponents("RTC", classes, 0);
	}
}
//---------------------------------------------------------------------------
